Package: lfl
Type: Package
Title: Linguistic Fuzzy Logic
Version: 2.4.0
Date: 2026-07-03
Authors@R: 
    person(given = "Michal",
           family = "Burda",
           role = c("aut", "cre"),
           email = "michal.burda@osu.cz",
           comment = c(ORCID = "0000-0002-4182-4407"))
Maintainer: Michal Burda <michal.burda@osu.cz>
Description: Various algorithms related to linguistic fuzzy logic: mining for
    linguistic fuzzy association rules, composition of fuzzy relations,
    performing Mamdani, implicative, and perception-based logical deduction
    (PbLD), and forecasting time-series using fuzzy rule-based ensemble (FRBE).
    The package also contains basic fuzzy-related algebraic functions capable of
    handling missing values in different styles (Bochvar, Sobocinski, Kleene
    etc.), computation of Sugeno integrals and the fuzzy transform.
License: GPL-3
Suggests: testthat, doMC, knitr, rmarkdown, R.rsp
Depends: R (>= 3.6)
Imports: Rcpp (>= 0.12.12), foreach, forecast (>= 5.5), plyr, tseries,
        e1071, tibble
LinkingTo: Rcpp
NeedsCompilation: yes
Encoding: UTF-8
Language: en-US
VignetteBuilder: R.rsp
Config/roxygen2/version: 8.0.0
Packaged: 2026-07-03 07:15:20 UTC; michal
Author: Michal Burda [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-4182-4407>)
Repository: CRAN
Date/Publication: 2026-07-03 08:50:10 UTC
Built: R 4.5.2; aarch64-apple-darwin20; 2026-07-03 12:40:23 UTC; unix
Archs: lfl.so.dSYM
