# Generated by roxygen2: do not edit by hand

S3method("[",fsets)
S3method(as.ctx3,ctx3)
S3method(as.ctx3,ctx3bilat)
S3method(as.ctx3,ctx5)
S3method(as.ctx3,ctx5bilat)
S3method(as.ctx3,default)
S3method(as.ctx3bilat,ctx3)
S3method(as.ctx3bilat,ctx3bilat)
S3method(as.ctx3bilat,ctx5)
S3method(as.ctx3bilat,ctx5bilat)
S3method(as.ctx3bilat,default)
S3method(as.ctx5,ctx3)
S3method(as.ctx5,ctx3bilat)
S3method(as.ctx5,ctx5)
S3method(as.ctx5,ctx5bilat)
S3method(as.ctx5,default)
S3method(as.ctx5bilat,ctx3)
S3method(as.ctx5bilat,ctx3bilat)
S3method(as.ctx5bilat,ctx5)
S3method(as.ctx5bilat,ctx5bilat)
S3method(as.ctx5bilat,default)
S3method(as.data.frame,farules)
S3method(as.data.frame,fsets)
S3method(as.matrix,fsets)
S3method(as_tibble,fsets)
S3method(c,farules)
S3method(cbind,fsets)
S3method(fcut,data.frame)
S3method(fcut,default)
S3method(fcut,factor)
S3method(fcut,logical)
S3method(fcut,matrix)
S3method(fcut,numeric)
S3method(lcut,data.frame)
S3method(lcut,default)
S3method(lcut,factor)
S3method(lcut,logical)
S3method(lcut,matrix)
S3method(lcut,numeric)
S3method(plot,fsets)
S3method(print,algebra)
S3method(print,ctx3)
S3method(print,ctx3bilat)
S3method(print,ctx5)
S3method(print,ctx5bilat)
S3method(print,farules)
S3method(print,frbe)
S3method(print,fsets)
S3method(rbind,fsets)
S3method(str,fsets)
export("specs<-")
export("vars<-")
export(aggregateConsequents)
export(algebra)
export(allowed.lingexpr)
export(antecedents)
export(as.ctx3)
export(as.ctx3bilat)
export(as.ctx5)
export(as.ctx5bilat)
export(compose)
export(consequents)
export(ctx3)
export(ctx3bilat)
export(ctx5)
export(ctx5bilat)
export(defaultHedgeParams)
export(defuzz)
export(dragonfly)
export(equidist)
export(equifreq)
export(evalfrbe)
export(farules)
export(fcut)
export(fire)
export(frbe)
export(fsets)
export(ft)
export(ftinv)
export(goedel.biresiduum)
export(goedel.residuum)
export(goedel.tconorm)
export(goedel.tnorm)
export(goguen.biresiduum)
export(goguen.residuum)
export(goguen.tconorm)
export(goguen.tnorm)
export(hedge)
export(horizon)
export(infer)
export(invol.neg)
export(is.algebra)
export(is.ctx3)
export(is.ctx3bilat)
export(is.ctx5)
export(is.ctx5bilat)
export(is.farules)
export(is.frbe)
export(is.fsets)
export(is.ft)
export(is.specific)
export(kleene)
export(lcut)
export(lingexpr)
export(lowerBound)
export(lowerEst)
export(lukas.biresiduum)
export(lukas.residuum)
export(lukas.tconorm)
export(lukas.tnorm)
export(mase)
export(minmax)
export(mult)
export(nelson)
export(pbld)
export(perceive)
export(pgoedel.tconorm)
export(pgoedel.tnorm)
export(pgoguen.tconorm)
export(pgoguen.tnorm)
export(plukas.tconorm)
export(plukas.tnorm)
export(quantifier)
export(raisedcos)
export(raisedcosine)
export(rbcoverage)
export(reduce)
export(rmse)
export(searchrules)
export(slices)
export(smape)
export(sobocinski)
export(specs)
export(strict.neg)
export(sugeno)
export(triangle)
export(triangular)
export(upperBound)
export(vars)
importFrom(Rcpp,evalCpp)
importFrom(e1071,kurtosis)
importFrom(e1071,skewness)
importFrom(foreach,"%do%")
importFrom(foreach,"%dopar%")
importFrom(foreach,foreach)
importFrom(forecast,auto.arima)
importFrom(forecast,ets)
importFrom(forecast,forecast)
importFrom(forecast,rwf)
importFrom(forecast,thetaf)
importFrom(plyr,colwise)
importFrom(plyr,laply)
importFrom(stats,as.formula)
importFrom(stats,coef)
importFrom(stats,frequency)
importFrom(stats,lm)
importFrom(stats,model.matrix)
importFrom(stats,na.omit)
importFrom(stats,sd)
importFrom(stats,ts)
importFrom(stats,ts.plot)
importFrom(stats,weighted.mean)
importFrom(tibble,as_tibble)
importFrom(tseries,adf.test)
importFrom(utils,str)
useDynLib(lfl, .registration = TRUE)
