
<!-- README.md is generated from README.Rmd. Please edit that file -->

[![CRAN
status](https://www.r-pkg.org/badges/version/lfl)](https://CRAN.R-project.org/package=lfl)
[![codecov](https://codecov.io/gh/beerda/lfl/branch/master/graph/badge.svg)](https://app.codecov.io/gh/beerda/lfl)

# lfl

The lfl package provides various algorithms related to linguistic fuzzy
logic: mining for linguistic fuzzy association rules, composition of
fuzzy relations, performing perception-based logical deduction (PbLD),
and forecasting time-series using fuzzy rule-based ensemble (FRBE). The
package also contains basic fuzzy-related algebraic functions capable of
handling missing values in different styles (Bochvar, Sobocinski, Kleene
etc.), computation of Sugeno integrals and fuzzy transform.

## Documentation

The complete documentation of the package is [available in this
vignette](https://github.com/beerda/lfl/blob/master/vignettes/main.pdf).

## Installation

To install the stable version from CRAN, simply issue the following
command within your R session:

``` r
install.packages("lfl")
```

If you want to install the development version instead, type:

``` r
install.packages("devtools")
devtools::install_github("beerda/lfl")
```
