# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

.goedel.tnorm <- function(vals) {
    .Call(`_lfl_goedel_tnorm`, vals)
}

.pgoedel.tnorm <- function(list) {
    .Call(`_lfl_pgoedel_tnorm`, list)
}

.lukas.tnorm <- function(vals) {
    .Call(`_lfl_lukas_tnorm`, vals)
}

.plukas.tnorm <- function(list) {
    .Call(`_lfl_plukas_tnorm`, list)
}

.goguen.tnorm <- function(vals) {
    .Call(`_lfl_goguen_tnorm`, vals)
}

.pgoguen.tnorm <- function(list) {
    .Call(`_lfl_pgoguen_tnorm`, list)
}

.goedel.tconorm <- function(vals) {
    .Call(`_lfl_goedel_tconorm`, vals)
}

.pgoedel.tconorm <- function(list) {
    .Call(`_lfl_pgoedel_tconorm`, list)
}

.lukas.tconorm <- function(vals) {
    .Call(`_lfl_lukas_tconorm`, vals)
}

.plukas.tconorm <- function(list) {
    .Call(`_lfl_plukas_tconorm`, list)
}

.goguen.tconorm <- function(vals) {
    .Call(`_lfl_goguen_tconorm`, vals)
}

.pgoguen.tconorm <- function(list) {
    .Call(`_lfl_pgoguen_tconorm`, list)
}

.goedel.residuum <- function(x, y) {
    .Call(`_lfl_goedel_residuum`, x, y)
}

.lukas.residuum <- function(x, y) {
    .Call(`_lfl_lukas_residuum`, x, y)
}

.goguen.residuum <- function(x, y) {
    .Call(`_lfl_goguen_residuum`, x, y)
}

.invol.neg <- function(x) {
    .Call(`_lfl_invol_neg`, x)
}

.strict.neg <- function(x) {
    .Call(`_lfl_strict_neg`, x)
}

.triangle <- function(x, ctx) {
    .Call(`_lfl_triangle`, x, ctx)
}

.raisedcos <- function(x, ctx) {
    .Call(`_lfl_raisedcos`, x, ctx)
}

.hedge <- function(x, p) {
    .Call(`_lfl_hedge`, x, p)
}

.mult <- function(x, y, f) {
    .Call(`_lfl_mult`, x, y, f)
}

.reduce <- function(config) {
    .Call(`_lfl_reduce`, config)
}

.search <- function(rcppData, config) {
    .Call(`_lfl_search`, rcppData, config)
}

.specificity <- function(x, y, vars, specs) {
    .Call(`_lfl_specificity`, x, y, vars, specs)
}

