% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lfl-package.R
\docType{package}
\name{lfl-package}
\alias{lfl}
\alias{lfl-package}
\title{lfl - Linguistic Fuzzy Logic}
\description{
Various algorithms related (not only) to linguistic fuzzy logic: mining for linguistic fuzzy association
rules, composition of fuzzy relations, performing perception-based logical deduction (PbLD),
and forecasting time-series using fuzzy rule-based ensemble (FRBE).
}
\details{
Other methods include fuzzy transform and computation of Sugeno integrals. Also, basic algebraic
functions related to fuzzy logic are included, which allow handling missing values using different
styles such as Kleene, Bochvar, Sobocinski and others.
}
\author{
\strong{Maintainer}: Michal Burda \email{michal.burda@osu.cz} (\href{https://orcid.org/0000-0002-4182-4407}{ORCID})

Authors:
\itemize{
  \item Michal Burda \email{michal.burda@osu.cz} (\href{https://orcid.org/0000-0002-4182-4407}{ORCID})
}

}
\keyword{internal}
